/****************************************************
* Dmodulation CLIP avec MSP430F1132                *
****************************************************/

/*------------------------------------------------------------------------------------
  Nom          :  CLIP_1232_F11.c
  Description  :  Programme de test des fonctions de gestion de l'alimentation
                  Pour carte CLIP de CREMMEL M. avec LCD Nokia 3310
  Author       :  12/05/2006 - CREMMEL Marcel
  History      :  12/05/2006 - Premire version
--------------------------------------------------------------------------------------*/
#include <msp430x12x2.h>

/*------------------------------------------------------------------------------------
                                         Constantes
--------------------------------------------------------------------------------------*/

/*------------------
       Cblage
--------------------*/
// Ports SPI
#define DIN_SPI     BIT1  //  P3.1 = SOMI = SDIN
#define CLK_SPI     BIT3  //  P3.3 = UCLK = SCLK

// Connexions LCD Nokia (transmission via SPI)
#define Lcd_CE_Pin  BIT0  //  P3.0 = CE_LCD
#define Lcd_DC_Pin  BIT2  //  P3.2 = D/C
#define Lcd_RST_Pin BIT2  //  P2.2 = RESET_LCD

// Connexions CNA MAX550BCPA (transmission via SPI)
#define CS_CNA   BIT6   // P3.6 = CS_CNA, actif  "0"
#define LDAC     BIT4   // P3.4 = LDAC, actif  "0"

// Sortie RXD dmodule
#define RXD      BIT7   // P3.7 = sortie dmodule RXD

// Autre signaux
#define PWR_ON   BIT0   // P2.0 = PWR_ON (alim Aops)
#define SONNE    BIT5   // P2.5 = SONNE (entre dtection de sonnerie)

// Signaux de test
#define TA1      BIT2   // P1.2 = signal de test TA1

/*--------------------
  Constantes Timer A
----------------------*/
// Interruption Timer A, voie 0
#define Pulse_OUT1      100     //Dure de l'impulsion de dclenchement du CAN
#define Periode_IntTA0  588     //Soit une frquence de 6800Hz avec l'horloge de 4MHZ

/*------------------------------------------------------------------------------------
                                    Variables globales
-------------------------------------------------------------------------------------*/
char Indicateurs; //Indicateurs de fonctionnement
#define WAIT_SMMR  1  // Bit 0 : indique la phase d'attente du signal SMMR
#define WAIT_MARK  2  // Bit 1 : indique la phase d'attente du signal MARK
#define WAIT_MSG   4  // Bit 2 : indique la phase de rception du message
#define NEW_MSG    8  // Bit 3 : indique un nouveau message
#define TIME_OUT   16 // Bit 4 : indique un problme pdt la rception de la trame
/*-------------------------
  Compteurs d'interruptions
---------------------------*/
int Compt_Sonnerie; // Pour dclencher la procdure de rception de la trame CLIP
                    // qu'aprs un signal de sonnerie
int Compt_Int_6800; // Permet d'arrter la procdure de rception de la trame CLIP
                    // en cas de problme (pas de trame, perturbations diverses, ...)
/*-------------------------
     Autres variables
---------------------------*/

/*------------------------------------------------------------------------------------
                                    Fonctions logicielles
-------------------------------------------------------------------------------------*/
/*---------------------------------------------------------
                       HORLOGES
-----------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  SetDCOClock
  Description  :  Affectation frquence DCO
  Arguments    :  DCO : valeur des 3 bits DCO
                  RES : valeur des 3 bits RESEL
  Valeur renvoye : aucune.
----------------------------------------------------*/
void SetDCOClock(int DCO,int RES)
{
  DCOCTL=(DCOCTL & ~(DCO2|DCO1|DCO0)) | (DCO << 5); //Affectation DCO[2..0]
  BCSCTL1=(BCSCTL1 & ~(RSEL2|RSEL1|RSEL0)) | RES  ; //Affectation RESEL[2..0]
}

/*---------------------------------------------------------
                          TIMER A
-----------------------------------------------------------*/

/*--------------------------------------------------
  Nom          :  InitTimerA_FSK
  Description  :  Initialisation du Timer A pour dmodulation FSK :
                    - TAR : continous up, horloge = ACLK, pas d'int. TAIE
                    - CCR0 : Output Compare
                       - interruptions dclenches par EQU0 valides
                       - TACCR0 affect ds le programme d'int pour obtenir une priode
                         de EQU0 gale  Periode_IntTA0
                    - CCR1 : Output Compare
                       - OUT1 : Mode Reset/Set (Set avec EQU0 et Reset avec EQU1)
                       - TACCR1 affect ds le programme d'int pour obtenir une dure
                         de l'impulsion OUT1 gale  Pulse_OUT1
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_FSK(void)
{
 BCSCTL1|=XTS; //LFXT1 : haute frquence (quartz 4MHz)
 TACTL=TASSEL0|MC0;  // Horloge TAR=ACLK=4MHz,
                     // Up mode modulo TACCR0
                     // Pas d'int TAIE
 TACCTL0=CCIS1|CCIE; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE=0 (non utilis)
                     // CCIE=1 : interruption EQU0 valide
 TACCR0=Periode_IntTA0-1; //Priodes int TA0 = SMCLK/PeriodeIntTA0 = 6800Hz environ
 TACCTL1=CCIS1|OUTMOD1|OUTMOD0; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE="011" : output Compare, OUT1 : Set/Reset
 TACCR1=Periode_IntTA0-1-Pulse_OUT1;//Pour produire une impulsion sur OUT1
                                    //de dure Pulse_OUT1
// P1SEL|=BIT2; //Test : P1.2/TA1=OUT1 pour controle  l'oscilloscope
// P1DIR|=BIT2;
}

/*--------------------------------------------------
  Nom          :  InitTimerA_Delay
  Description  :  Initialisation du Timer A pour gnrer des dlais :
                    - TAR : continous up, horloge = FDCO/8, int. TAIE inhibe
                    - CCR0, CCR1 et CCR2 : non utiliss
  Arguments    :  aucun
  Valeur renvoye : aucune.
----------------------------------------------------*/
void InitTimerA_Delay(void)
{
 BCSCTL1&=~XTS; //LFXT1 : basse frquence pour rduire la consommation
 TACTL=TASSEL1|ID1|ID0|MC1;  // Horloge TAR=SMCLK/8=FDCO/8,
                             // mode continu, int TAIE inhibe
 TACCTL0=0 ;//Aucune fonction
 TACCTL2=0 ;//Aucune fonction
 TACCTL1=0 ;//Aucune fonction
// TACCTL1=CCIS1|OUTMOD1|OUTMOD0; // Input capture = GND
                     // CAP=0 : compare mode
                     // OUTMODE="011" : output Compare, OUT1 : Set/Reset
// TACCR1=65536/4 ;//Pour produire un signal de rapport cyclique 1/4
// P1SEL|=BIT2; //Test : P1.2/TA1=OUT1 pour controle  l'oscilloscope
// P1DIR|=BIT2;
}

/*---------------------------------------------------------
                       Fonctions diverses
-----------------------------------------------------------*/

/*------------------------------------------------------------------------------------
                                       Programme principal
--------------------------------------------------------------------------------------*/
void main(void)
{
 WDTCTL=WDTPW+WDTHOLD;       // Arrt du chien de garde
 // Horloge basse
 SetDCOClock(3,1); // FDCO = 190kHz typique
 // Initialisations ports E/S
 P1DIR=TA1; // Port 1
 P2DIR=Lcd_RST_Pin|PWR_ON; // Port 2
 P3DIR=Lcd_CE_Pin|DIN_SPI|CLK_SPI|Lcd_DC_Pin|RXD|CS_CNA|LDAC;//Port 3
 P3OUT=Lcd_CE_Pin|CS_CNA|LDAC;// Controleur LCD et CNA non slectionns
 P2OUT=Lcd_RST_Pin; // Pas de reset (RST = "1") et PWR_ON = "0"
 // Dtection sonnerie
 P2IE |=SONNE; //Validation interruption sur P2.5
 P2IES|=SONNE; //Sur flanc descendant
 // Initialisation TimerA pour raliser monostable (int inhibe)
 InitTimerA_Delay(); // Horloge TAR = FDCO/8 = 23750Hz typique
 Compt_Sonnerie=0; // Raz du compteur de priodes de sonnerie
 Indicateurs=0;    // Raz de tous les indicateurs
 // Validation interruptions ds CPU
 _EINT();
/*--------------------
   Boucle principale
 ---------------------*/
 do
  {
   P3OUT^=RXD ;// Test : inversion du bit RXD=P3.7
   /* Fonctions :
        - Si pas de nouveau message reu :
            - Arrt CPU hors interruptions :
               - Low power mode 4 (LPM4=tout  l'arrt) si sonnerie non dtecte
               - Low power mode 1 (LPM1) entre dtection sonnerie et rception trame CLIP :
                              - CPU et MCLK off
                              - DCO oprationnel car utilis (horloge TAR = FDCO/8)
        - Si nouveau message reu :
            - Horloge CPU basse
            - Affichage du numro sur le LCD */
   if (Indicateurs==0)
   {
    if (TACTL & TAIE) __low_power_mode_1();// CPU et MCLK off si dtection sonnerie
                 else __low_power_mode_4();// Tout  l'arrt
   }
   if (Indicateurs & NEW_MSG)
   {
    // Passer en mode "basse consommation" et affichage du numro appelant
    _DINT();
    SetDCOClock(3,1);  // Horloge basse : FDCO = 190kHz typique
    #define FDCO_low 190000
    P2OUT&=~PWR_ON;    // Arrt alimentation Aop
    InitTimerA_Delay();// Initialisation TimerA pour raliser monostable (int inhibe)
                       // Horloge TAR = FDCO/8 = 23750Hz typique
    if (!(Indicateurs & TIME_OUT))
     {
      // Affichage du numro reu sur le LCD
     }
    Indicateurs=0;
    _EINT();
   }
  }
 while (1);
}

/*------------------------------------------------------------------------------------
                                     Programmes d'interruption
--------------------------------------------------------------------------------------*/
/*-----------------------
   Interruption Port P2
 ------------------------
 Active  chaque flanc descendant sur P2.5=SONNE :
   - Prochaine interruption TAI ds 500mS (monostable redclenchable)
     (dure mini phase B de la transmission) */
 #pragma vector=PORT2_VECTOR
 __interrupt void Int_PA2()
 {
  Compt_Sonnerie++;// Incrmentation du compteur de priodes de sonnerie
  TAR=0xFFFF-FDCO_low/16; // Prochaine interruption TAI ds 500mS
  TACTL&=~TAIFG;   // Raz indicateur TAIFG
  TACTL|=TAIE;     // Validation interruption TAI
  P2IFG&=~SONNE;   // Acquitement interruption
  __low_power_mode_off_on_exit();
 }

/*-------------------
   Interruption TAI
 --------------------
 Active 500mS aprs arrt sonnerie puis inhibe
  - Activer la dmodulation FSK par interruptions TimerA au retour d'int. */
#pragma vector=TIMERA1_VECTOR
__interrupt void Int_TAI()
{
 if (Compt_Sonnerie > 10) // Nb de priodes de sonnerie suffisants
  {
   // Alimentation Aop
   P2OUT|=PWR_ON;
   // Horloge haute
   SetDCOClock(3,6); // FDCO = 1,9MHz typique
   // Raz compteur d'interruptions Timer A
   Compt_Int_6800=0;
   // Initialisation TimerA pour dmodulation FSK
   InitTimerA_FSK(); //Initialisation du Timer A (interruptions 6800Hz)
   TACTL&=~TAIE;     //Inhibition prochaines interruptions
   Indicateurs=WAIT_SMMR; //Phase d'attente du signal SMMR
  }
 Compt_Sonnerie=0;
 TACTL&=~TAIFG;     //Raz indicateur TAIFG (acquitement interruption)
 __low_power_mode_off_on_exit();
}

/*-------------------
   Interruption TA0
 --------------------
 Fonction : dmodulation FSK
            test ici : incrmentation Compt_Int_6800
                       dtection "TimeOut (1s)       */
#pragma vector=TIMERA0_VECTOR
__interrupt void INTER_TIMERA0()
{
 // Lcd_DC_Pin utilis pour mesurer le temps d'excution
 // et la frquence du programme d'interruption
 P3OUT|=Lcd_DC_Pin;  // Lcd_DC_Pin="1"
 Compt_Int_6800++;
 if (Compt_Int_6800 > 6800) Indicateurs=TIME_OUT|NEW_MSG;
 P3OUT&=~Lcd_DC_Pin; // Lcd_DC_Pin="0"
}
